package cn.teammodel.controller.admin.service.impl;

import cn.teammodel.common.ErrorCode;
import cn.teammodel.common.PK;
import cn.teammodel.config.exception.ServiceException;
import cn.teammodel.controller.admin.service.CommonService;
import cn.teammodel.controller.admin.service.MoralEducationService;
import cn.teammodel.model.dto.admin.common.GCDto;
import cn.teammodel.model.dto.admin.common.GroupDto;
import cn.teammodel.model.dto.admin.common.RGroupList;
import cn.teammodel.model.dto.admin.common.RMember;
import cn.teammodel.model.dto.admin.labor.FindDto;
import cn.teammodel.model.dto.admin.labor.LaborDto;
import cn.teammodel.model.entity.appraise.Appraise;
import cn.teammodel.model.entity.appraise.AppraiseTreeNode;
import cn.teammodel.model.entity.common.Exam;
import cn.teammodel.model.entity.common.ExamClassResult;
import cn.teammodel.model.entity.school.ClassInfo;
import cn.teammodel.model.entity.school.LessonRecord;
import cn.teammodel.model.entity.school.School;
import cn.teammodel.model.vo.Common.ExamVo;
import cn.teammodel.model.vo.admin.GradeAndClassVo;
import cn.teammodel.model.vo.appraise.RecordVo;
import cn.teammodel.repository.*;
import cn.teammodel.security.utils.SecurityUtil;
import cn.teammodel.service.impl.EvaluationServiceImpl;
import cn.teammodel.test.LessonRecordQueryService;
import cn.teammodel.utils.GroupUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import lombok.AllArgsConstructor;
import lombok.Data;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

@Service
public class MoralEducationServiceImpl implements MoralEducationService {

    @Resource
    private LessonRecordRepository lessonRecordRepository;
    @Resource
    private ClassRepository classRepository;
    @Resource
    private AppraiseRecordRepository appraiseRecordRepository;
    @Resource
    private ExamRepository examRepository;
    @Resource
    private ExamClassResultRepository examClassResultRepository;
    @Resource
    private AppraiseRepository appraiseRepository;
    @Resource
    private EvaluationServiceImpl evaluationService;
    @Resource
    private SchoolRepository schoolRepository;
    @Resource
    private CommonService commonService;

    private static Environment environment; // 静态字段
    @Autowired
    private Environment env; // 非静态字段

    @PostConstruct
    public void init() {
        MoralEducationServiceImpl.environment = env; // 在初始化时将非静态字段赋值给静态字段
    }
    @Override
    public Map<String, Object> getAnalysis(LaborDto laborDto, HttpServletRequest request) {

        //根据具体参数查询相关课列内容
        List<LessonRecord> records;
        LessonRecordQueryService queryService = new LessonRecordQueryService(lessonRecordRepository);
        String schoolId = SecurityUtil.getLoginUser().getSchoolId();
        String lessonRecordKey = String.format(PK.PK_LESSON_RECORD, laborDto.getCode());
        Long startTime = laborDto.getStartTime();
        Long endTime = laborDto.getEndTime();
        String subjectId = laborDto.getSubjectId();
        String tmdId = laborDto.getTmdId();
        String grade = laborDto.getGrade();
        String periodId = laborDto.getPeriodId();
        String academicYearId = laborDto.getAcademicYearId();
        try {

            records = queryService.queryLessonsInParallel(
                    lessonRecordKey, startTime, endTime, subjectId, null, grade, periodId
            );
            // 获取当前月份的起始时间和结束时间
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            long startOfMonth = calendar.getTimeInMillis();

            calendar.add(Calendar.MONTH, 1);
            calendar.add(Calendar.MILLISECOND, -1);
            long endOfMonth = calendar.getTimeInMillis();

            // 筛选出本月的数据量
            int currentRecordsCount = records.stream()
                    .filter(record -> record.getStartTime() >= startOfMonth && record.getStartTime() <= endOfMonth)
                    .mapToInt(record -> 1)
                    .sum();

        } catch (InterruptedException | ExecutionException e) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据查询异常");
        } finally {
            queryService.shutdown();
        }
        //总评价数
        List<ClassInfo> classes = classRepository.findClassBySchoolIdAndPeriodId(laborDto.getPeriodId(), String.format(PK.CLASS, schoolId));
        if (classes.isEmpty()) {
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "暂无班级");
        }
        List<String> classIds = classes.stream().map(ClassInfo::getId).collect(Collectors.toList());
        List<RecordVo> res = appraiseRecordRepository.latestRecords(
                String.format(PK.PK_APPRAISE_RECORD, schoolId),
                academicYearId,
                startTime,
                endTime,
                "劳育",
                classIds
        );
        //表扬的次数
        int rightCount = (int) res.stream().filter(RecordVo::isPraise).count();
        //批评的次数
        int wrongCount = (int) res.stream().filter(record -> !record.isPraise()).count();
        //已经评价的学生人数
        long uniqueTargetIdCount = res.stream()
                .map(RecordVo::getTargetId)
                .filter(Objects::nonNull) // 过滤掉 null 值
                .distinct()
                .count();

        List<String> recordIds = new ArrayList<>();
        for (LessonRecord record : records) {
            recordIds.add(record.getId());
        }
        //examIds.add("c26e5766-597f-45b3-91ad-19e76426323b");
        //获取所有课程下的课中活动
        List<ExamVo> exams = new ArrayList<>();
        if (!recordIds.isEmpty()) {
            exams = examRepository.findExamsByIds(laborDto.getSource(),recordIds);
            exams = exams.stream()
                    .filter(exam -> !exam.getClasses().isEmpty())
                    .collect(Collectors.toList());
        }
        Map<String, List<List<String>>> knowledgeMap = new HashMap<>();
        Map<String, List<Double>> point = new HashMap<>();
        List<String> examIds = new ArrayList<>();
        for (ExamVo exam : exams) {
            examIds.add(exam.getId());
            String lessonRecordId = exam.getLessonRecordId(); // 获取 lessonRecordId
            Optional<LessonRecord> recordOptional = records.stream()
                    .filter(record -> record.getId().equals(lessonRecordId))
                    .findFirst();
            if (recordOptional.isPresent()) {
                LessonRecord record = recordOptional.get();
                exam.setRecordName(record.getName()); // 设置 recordName
            }
            if (exam.getPapers() != null) {
                if (exam.getPapers().get(0).getKnowledge() != null)
                {
                    knowledgeMap.put(exam.getId(), exam.getPapers().get(0).getKnowledge());
                    point.put(exam.getId(), exam.getPapers().get(0).getPoint());
                }
            }
        }
        List<ExamClassResult> examResults = new ArrayList<>();
        if (!examIds.isEmpty()) {
            examResults = examClassResultRepository.findAll(String.format(PK.CLASS_RESULT, laborDto.getCode()),examIds);
        }
        //获取学校基础信息
        List<School.Period> period = schoolRepository.findPeriodById(schoolId,periodId);

        List<GradeAndClassVo> gradeAndClassVos = commonService.getGradeAndClass(new GCDto(schoolId, periodId));
        Map<String, Object> resMap = new HashMap<>();
        //处理劳育知识块和知识点的关联关系
        resMap.put("exams",exams);//评测明细
        resMap.put("subjectiveCount", res.size()); //主观评价数
        resMap.put("examCount", exams.size());//评测数量
        resMap.put("lessonCount", records.size());//班会课数量
        resMap.put("rightCount", rightCount);//表扬评价数
        resMap.put("wrongCount", wrongCount);//待改进评价数
        resMap.put("uniqueTargetIdCount", uniqueTargetIdCount);//评价学生数
        Appraise appraise = appraiseRepository.findAppraiseBySchoolIdAndPeriodIdAndCode(schoolId, periodId, PK.PK_APPRAISE);
        if (appraise != null) {
            appraise = evaluationService.buildTree(appraise);
            Map<String, List<String>> knowledgeBlockToPointsMap = getKnowledgeBlockToPointsMap(appraise);
            Map<String, Object> classScoreRate = new HashMap<>();
            Map<String, Object> gradeScoreRate = new HashMap<>();
            if (laborDto.getClassId() != null) {
                classScoreRate =  calculateKnowledgeScoreRateForClass(laborDto.getClassId(), examResults, knowledgeMap, appraise, point,res,request);
            }
            if (laborDto.getGrade() != null) {
                gradeScoreRate = calculateKnowledgeScoreForGrade(laborDto.getGrade(), examResults, knowledgeMap,appraise, point,res,gradeAndClassVos);
            }
            Map<String,Object> schoolScoreRate = calculateKnowledgeScoreForSchool(examResults, knowledgeMap,appraise, point,res,period,gradeAndClassVos);
            resMap.put("gradeScoreRate", gradeScoreRate);
            resMap.put("classScoreRate", classScoreRate);
            resMap.put("schoolScoreRate", schoolScoreRate);
            List<Map<String, Object>>  scores = calculateScoresWithDetails(res, appraise);
            List<Map<String, Object>> students = combineScoresWithExamResults(scores,examResults,knowledgeMap,point,knowledgeBlockToPointsMap);
            resMap.put("scores", students);
        }
        //处理主观评价内容
        return resMap;

    }

    @Override
    public Map<String, Object> getDetails(FindDto findDto, HttpServletRequest request) {
        Map<String, Object> resMap = new HashMap<>();
        List<Exam> exams;
        List<RecordVo> res;
        try {
            exams = examRepository.findExamByClassId(String.format(PK.EXAM, findDto.getTmdId()), findDto.getClassId(), findDto.getPeriodId(), findDto.getSubjectId());
            res= appraiseRecordRepository.getRecords(String.format(PK.PK_APPRAISE_RECORD, findDto.getCode()),findDto.getAcademicYearId(), findDto.getClassId(), findDto.getStudentId());
            resMap.put("exams",exams);
            resMap.put("res", res);
        }catch (Exception e){
            throw new ServiceException(ErrorCode.SYSTEM_ERROR.getCode(), "数据查询异常");
        }
        return resMap;
    }

    public List<Map<String, Object>> combineScoresWithExamResults(
            List<Map<String, Object>> scores,
            List<ExamClassResult> examResults,
            Map<String, List<List<String>>> knowledgeMap,
            Map<String, List<Double>> points,
            Map<String, List<String>> knowledgeBlockToPointsMap) {

        // 1. 计算每个学生的知识点得分
        Map<String, Map<String, Double>> studentKnowledgePointScores = new HashMap<>();
        for (Map<String, Object> studentResult : scores) {
            String studentId = (String) studentResult.get("studentId");

            // 初始化知识点总分
            Map<String, Double> knowledgeTotalScore = new HashMap<>();

            // 计算当前学生的知识点得分
            calculateStudentScoreRates(studentId, examResults, knowledgeMap, points, knowledgeTotalScore);

            // 将知识点得分存储到 studentKnowledgePointScores 中
            studentKnowledgePointScores.put(studentId, knowledgeTotalScore);
        }

        // 2. 将知识点得分汇总到知识块级别
        Map<String, Map<String, Double>> studentKnowledgeBlockScores = getStringMapMap(knowledgeBlockToPointsMap, studentKnowledgePointScores);

        // 3. 更新现有的 scores 返回数据
        for (Map<String, Object> studentResult : scores) {
            String studentId = (String) studentResult.get("studentId");
            Map<String, Integer> subjectiveScores = (Map<String, Integer>) studentResult.get("scores");

            // 查找该学生的知识块客观成绩
            Map<String, Double> knowledgeBlockScores = studentKnowledgeBlockScores.get(studentId);

            // 初始化主观总分和客观总分
            double subjectiveTotal = 0.0;
            double objectiveTotal = 0.0;
            int knowledgeBlockCount = knowledgeBlockScores.size();

            // 更新主观成绩，计算主观总分和客观总分
            Map<String, Object> combinedScores = new HashMap<>();
            for (Map.Entry<String, Integer> entry : subjectiveScores.entrySet()) {
                double score = 0;
                String knowledgeBlock = entry.getKey();
                int count = entry.getValue(); // 次数
                // 确保 count 不小于0
                if (count < 0) {
                    count = 0;
                }
                double maxCount = 50.0; // 调整此参数控制分数增长速率

                // 使用反比例函数计算加分
                double additionalScore = 40.0 * ((double) count / (count + maxCount));
                double convertedScore = 60.0 + additionalScore;

                // 保留两位小数
                convertedScore = Double.parseDouble(String.format("%.2f", convertedScore));
                subjectiveTotal += convertedScore;

                // 获取对应知识块的客观成绩，并转换为基于60分加分的分数
                double objectiveScore = knowledgeBlockScores.get(knowledgeBlock);
                // 转换为正确率的小数形式（假设原始分是百分比）
                double correctRate = objectiveScore / 100.0;
                objectiveScore = 60.0 + correctRate * 40.0;
                // 确保不超过100分
                objectiveScore = Math.min(100.0, objectiveScore);
                // 保留两位小数
                objectiveScore = Double.parseDouble(String.format("%.2f", objectiveScore));
                objectiveTotal += objectiveScore;

                // 计算综合得分（主观占60%，客观占40%）
                score = convertedScore * 0.6 + objectiveScore * 0.4;
                score = Double.parseDouble(String.format("%.2f", score));

                // 存储结果
                double finalConvertedScore = convertedScore;
                double finalObjectiveScore = objectiveScore;
                double finalScore = score;
                combinedScores.put(knowledgeBlock, new HashMap<String, Object>() {{
                    put("count", finalConvertedScore);
                    put("objectiveScore", finalObjectiveScore);
                    put("compositeScore", finalScore);
                }});
            }

            // 计算客观平均分
            double objectiveAverage = (knowledgeBlockCount > 0) ? objectiveTotal / knowledgeBlockCount : 0.0;
            objectiveAverage = Double.parseDouble(String.format("%.2f", objectiveAverage));

            // 计算主观平均分
            double subjectiveAverage = (subjectiveTotal > 0) ? subjectiveTotal / knowledgeBlockCount : 0.0;
            subjectiveAverage = Double.parseDouble(String.format("%.2f", subjectiveAverage));

            // 计算综合得分（主观占60%，客观占40%）
            double compositeScore = (subjectiveAverage * 0.6) + (objectiveAverage * 0.4);
            compositeScore = Double.parseDouble(String.format("%.2f", compositeScore));

            // 更新返回数据
            studentResult.put("scores", combinedScores);
            studentResult.put("subjectiveTotal", subjectiveAverage);
            studentResult.put("objectiveTotal", objectiveAverage);
            studentResult.put("compositeScore", compositeScore);
        }

        return scores;
    }

    private static @NotNull Map<String, Map<String, Double>> getStringMapMap(Map<String, List<String>> knowledgeBlockToPointsMap, Map<String, Map<String, Double>> studentKnowledgePointScores) {
        Map<String, Map<String, Double>> studentKnowledgeBlockScores = new HashMap<>();
        for (Map.Entry<String, Map<String, Double>> entry : studentKnowledgePointScores.entrySet()) {
            String studentId = entry.getKey();
            Map<String, Double> pointScores = entry.getValue();
            Map<String, Double> blockScores = studentKnowledgeBlockScores.computeIfAbsent(studentId, k -> new HashMap<>());

            // 遍历知识块和知识点的映射关系
            for (Map.Entry<String, List<String>> blockEntry : knowledgeBlockToPointsMap.entrySet()) {
                String knowledgeBlock = blockEntry.getKey();
                List<String> knowledgePoints = blockEntry.getValue();

                // 计算该知识块的总得分和知识点数量
                double totalScore = 0.0;
                int validPointCount = 0; // 有效知识点数量（有实际得分的知识点）
                for (String knowledgePoint : knowledgePoints) {
                    if (pointScores.containsKey(knowledgePoint)) {
                        totalScore += pointScores.get(knowledgePoint);
                        validPointCount++;
                    }
                }

                // 计算知识块的平均分
                double averageScore = (validPointCount > 0) ? totalScore / validPointCount : 60.0; // 无有效知识点时默认60分
                averageScore = Math.max(60.0, Math.min(100.0, averageScore)); // 确保得分在60-100之间
                blockScores.put(knowledgeBlock, averageScore);
            }
        }
        return studentKnowledgeBlockScores;
    }


    public static List<Map<String, Object>> calculateScoresWithDetails(List<RecordVo> res, Appraise appraise) {
        // 1. 构建知识点到知识块的映射 (一个知识块对应多个知识点)
        Map<String, List<String>> knowledgeBlockToPointsMap = getKnowledgeBlockToPointsMap(appraise);

        // 2. 构建所有二级知识块名称列表
        Set<String> knowledgeBlocks = new HashSet<>();
        for (AppraiseTreeNode node : appraise.getNodes()) {
            if ("劳育".equals(node.getName())) {
                for (AppraiseTreeNode secondLevelNode : node.getChildren()) {
                    knowledgeBlocks.add(secondLevelNode.getName());
                }
            }
        }

        // 3. 构建知识点到知识块的映射 (一个知识点对应一个知识块)
        Map<String, String> knowledgePointToBlockMap = new HashMap<>();
        for (Map.Entry<String, List<String>> entry : knowledgeBlockToPointsMap.entrySet()) {
            String knowledgeBlock = entry.getKey();
            for (String knowledgePoint : entry.getValue()) {
                knowledgePointToBlockMap.put(knowledgePoint, knowledgeBlock);
            }
        }

        // 4. 遍历评价记录并统计得分
        Map<String, Map<String, Integer>> studentScores = new HashMap<>();
        Map<String, List<Map<String, Object>>> studentAppraises = new HashMap<>(); // 存储每个学生的评价记录

        for (RecordVo record : res) {
            String studentId = record.getTargetId(); // 学生 ID
            String studentName = record.getTargetName(); // 学生名称
            String className = record.getClassName(); // 班级名称
            String appraiseName = record.getAppraiseName(); // 评价内容
            boolean isPraise = record.isPraise(); // 是否为优点

            // 获取评价内容对应的第二级节点（知识块）
            String knowledgeBlock = knowledgePointToBlockMap.get(appraiseName);
            if (knowledgeBlock == null) {
                continue; // 如果不在劳育知识块中，跳过
            }

            // 初始化学生得分记录
            String studentKey = studentId + "|" + studentName + "|" + className; // 组合学生唯一标识
            studentScores.putIfAbsent(studentKey, new HashMap<>());
            Map<String, Integer> studentScoreMap = studentScores.get(studentKey);

            // 计分
            int score = isPraise ? 1 : -1;
            int currentScore = studentScoreMap.getOrDefault(knowledgeBlock, 0) + score;
            if (currentScore < 0) {
                currentScore = 0; // 或者你可以设置为其他最小允许的分数
            }
            studentScoreMap.put(knowledgeBlock, currentScore);

            // 存储每个学生的评价记录
            studentAppraises.putIfAbsent(studentKey, new ArrayList<>());
            Map<String, Object> appraiseRecord = new HashMap<>();
            appraiseRecord.put("appraiseName", appraiseName);
            appraiseRecord.put("isPraise", isPraise);
            studentAppraises.get(studentKey).add(appraiseRecord);
        }

        // 5. 初始化缺失的二级知识块分数
        for (Map.Entry<String, Map<String, Integer>> entry : studentScores.entrySet()) {
            Map<String, Integer> studentScoreMap = entry.getValue();
            for (String block : knowledgeBlocks) {
                studentScoreMap.putIfAbsent(block, 0);
            }
        }

        // 6. 整理并返回结果
        return getMaps(studentScores, studentAppraises);
    }

    // 整理并返回结果
    private static List<Map<String, Object>> getMaps(Map<String, Map<String, Integer>> studentScores, Map<String, List<Map<String, Object>>> studentAppraises) {
        List<Map<String, Object>> result = new ArrayList<>();

        for (Map.Entry<String, Map<String, Integer>> entry : studentScores.entrySet()) {
            String studentKey = entry.getKey();
            Map<String, Integer> scoreMap = entry.getValue();
            List<Map<String, Object>> appraiseList = studentAppraises.get(studentKey); // 获取学生的评价记录

            // 解析学生唯一标识
            String[] studentInfo = studentKey.split("\\|");
            String studentId = studentInfo[0];
            String studentName = studentInfo[1];
            String className = studentInfo[2];

            // 构建返回结果
            Map<String, Object> studentResult = new HashMap<>();
            studentResult.put("studentId", studentId);
            studentResult.put("studentName", studentName);
            studentResult.put("className", className);
            studentResult.put("scores", scoreMap);
            studentResult.put("appraises", appraiseList); // 添加评价记录

            result.add(studentResult);
        }

        return result;
    }

    // 构建知识块到知识点的映射 (一个知识块对应多个知识点)
    private static Map<String, List<String>> getKnowledgeBlockToPointsMap(Appraise appraise) {
        Map<String, List<String>> knowledgeBlockToPointsMap = new HashMap<>();

        for (AppraiseTreeNode node : appraise.getNodes()) {
            if ("劳育".equals(node.getName())) {
                for (AppraiseTreeNode secondLevelNode : node.getChildren()) {
                    String knowledgeBlock = secondLevelNode.getName();
                    List<String> knowledgePoints = new ArrayList<>();

                    // 假设知识点的信息存储在 secondLevelNode.getChildren() 中
                    for (AppraiseTreeNode pointNode : secondLevelNode.getChildren()) {
                        knowledgePoints.add(pointNode.getName());
                    }

                    knowledgeBlockToPointsMap.put(knowledgeBlock, knowledgePoints);
                }
            }
        }

        return knowledgeBlockToPointsMap;
    }


    // 知识点得分率结果类
    @Data
    @AllArgsConstructor
    public static class KnowledgeScoreRate {
        public String knowledge; // 知识点
        public double scoreRate; // 得分率
    }

    /**
     * 计算某个学生在多次考试中的知识点得分率，并返回其所属节点的父节点得分率
     *
     * @param studentId       学生 ID
     * @param examResults     学生作答结果集
     * @param knowledgeMap    知识点映射（试卷 ID -> 知识点列表）
     * @param appraise        评价对象
     * @return 父节点得分率
     */
    public static Map<String, Double> calculateKnowledgeScoreRateForStudent(
            String studentId, List<ExamClassResult> examResults,
            Map<String, List<List<String>>> knowledgeMap, Appraise appraise, Map<String, List<Double>> points) {
        Map<String, Double> knowledgeTotalScore = new HashMap<>();

        // 计算学生的知识点得分率
        List<LaborEducationServiceImpl.KnowledgeScoreRate> studentScoreRates = calculateStudentScoreRates(studentId, examResults, knowledgeMap, points, knowledgeTotalScore);

        // 查找知识点对应的节点及其父节点
        Map<String, Double> parentNodeScoreRates = new HashMap<>();
        Map<String, Integer> parentNodeScoreCount = new HashMap<>(); // 记录每个父节点的知识点数量

        for (LaborEducationServiceImpl.KnowledgeScoreRate scoreRate : studentScoreRates) {
            AppraiseTreeNode node = findKnowledgeNode(appraise.getNodes(), scoreRate.getKnowledge());
            if (node != null) {
                AppraiseTreeNode parentNode = findParentNode(appraise.getNodes(), node.getId());
                if (parentNode != null) {
                    // 累加父节点的得分
                    double currentScore = parentNodeScoreRates.getOrDefault(parentNode.getName(), 0.0);
                    parentNodeScoreRates.put(parentNode.getName(), currentScore + scoreRate.getScoreRate());

                    // 累加父节点的知识点数量
                    int currentCount = parentNodeScoreCount.getOrDefault(parentNode.getName(), 0);
                    parentNodeScoreCount.put(parentNode.getName(), currentCount + 1);
                } else {
                    // 如果没有找到父节点，直接使用当前知识点的得分
                    parentNodeScoreRates.put(node.getName(), scoreRate.getScoreRate());
                    parentNodeScoreCount.put(node.getName(), 1);
                }
            }
        }

        // 计算父节点的平均得分（原始正确率）后，转换为基于60分的加分制
        parentNodeScoreRates.replaceAll((parentNodeName, totalScore) -> {
            int count = parentNodeScoreCount.getOrDefault(parentNodeName, 1);
            double averageScore = totalScore / count; // 原始正确率（0-100的百分比）

            // 将原始正确率转换为以60分为基础的加分制（正确率 × 40%）
            double correctRate = averageScore / 100.0;       // 转为0-1的小数
            double convertedScore = 60.0 + (correctRate * 40.0); // 最低60分，最高100分

            // 确保不超过100分，并保留两位小数
            convertedScore = Math.min(100.0, convertedScore);
            return Double.parseDouble(String.format("%.2f", convertedScore));
        });

        // 初始化所有同层的父节点
        for (AppraiseTreeNode node : appraise.getNodes()) {
            if (node.getName().equals("劳育")) {
                for (AppraiseTreeNode child : node.getChildren()) {
                    if (!parentNodeScoreRates.containsKey(child.getName())) {
                        parentNodeScoreRates.put(child.getName(), 60.0); // 默认60分
                    }
                }
            }
        }

        return parentNodeScoreRates;
    }


    /**
     * 计算班级每个学生知识点得分率，并返回每个学生所属节点的父节点得分率
     *
     * @param classId         班级 ID
     * @param examResults     学生作答结果集
     * @param knowledgeMap    知识点映射（试卷 ID -> 知识点列表）
     * @param appraise        评价对象
     * @return 学生 ID -> 父节点得分率映射
     */
    public static Map<String, Object> calculateKnowledgeScoreRateForClass(
            String classId,
            List<ExamClassResult> examResults,
            Map<String, List<List<String>>> knowledgeMap,
            Appraise appraise,
            Map<String, List<Double>> points,
            List<RecordVo> res,
            HttpServletRequest request) {

        // 将 classId 转换为 List<String>
        List<String> classIds = Collections.singletonList(classId);
        GroupDto groupDto = new GroupDto();
        groupDto.setIds(classIds);
        groupDto.setSchoolId(appraise.getSchoolId());
        String url = environment.getProperty("ies.server-url-group");
        Map<String, Object> groupId = GroupUtil.getGroupId(groupDto, new GroupUtil(environment), request, url);
        List<RGroupList> rGroupList = new ArrayList<>();
        List<RMember> rMembers = new ArrayList<>();
        for (Map.Entry<String, Object> entry : groupId.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            if (key.equals("groups")) {
                String jsonGroups = JSON.toJSONString(value);
                rGroupList = JSON.parseObject(jsonGroups, new TypeReference<List<RGroupList>>() {});
            }
            if (key.equals("members")) {
                String jsonGroups = JSON.toJSONString(value);
                rMembers = JSON.parseObject(jsonGroups, new TypeReference<List<RMember>>() {});
            }
        }
        String className = rGroupList.stream()
                .filter(rGroupList1 -> rGroupList1.getId().equals(classId))
                .findFirst()
                .map(RGroupList::getName)
                .orElse("未知班级"); // 如果未找到则返回默认值 "未知班级"

        // 1. 获取班级所有学生的客观分数
        Map<String, Map<String, Double>> studentScoreRates = new HashMap<>();
        Map<String, Double> classScoreRates = new HashMap<>();

        // 获取班级所有学生的名单（来自 rMembers）
        Map<String, String> studentIdToName = new HashMap<>();
        RGroupList groupList =  rGroupList.stream().filter(rGroupList1 -> rGroupList1.getId().equals(classId)).findFirst().orElse(null);
        if (groupList != null) {
            for (RMember member : groupList.members) {
                studentIdToName.put(member.getId(), member.getName());
            }
        }

        // 初始化知识块列表（从 appraise 配置中提取）
        Set<String> knowledgeBlocks = new HashSet<>();
        for (AppraiseTreeNode node : appraise.getNodes()) {
            if (node.getName().equals("劳育")) {
                for (AppraiseTreeNode child : node.getChildren()) {
                    knowledgeBlocks.add(child.getName());
                }
            }
        }

        if (examResults != null && !examResults.isEmpty()) {
            // 遍历班级中的每个学生
            for (ExamClassResult examResult : examResults) {
                if (!examResult.getInfo().getId().equals(classId)) continue; // 过滤出该班级的考试
                List<Integer> statuses = examResult.getStatus();
                for (int i = 0; i < examResult.getStudentIds().size(); i++) {
                    String studentId = examResult.getStudentIds().get(i);
                    String name = rMembers.stream()
                            .filter(member -> member.getId().equals(studentId))
                            .findFirst()
                            .map(RMember::getName)
                            .orElse("未知");
                    int status = statuses.get(i);
                    if (status == 1) continue; // 跳过 status 为 1 的记录

                    // 计算父节点得分率（客观分数）
                    Map<String, Double> scoreRates = calculateKnowledgeScoreRateForStudent(
                            studentId, examResults, knowledgeMap, appraise, points
                    );
                    scoreRates.replaceAll((k, v) ->
                            Math.min(100.0, Math.max(60.0, v))  // 60分打底，最高100分
                    );
                    studentScoreRates.put(name, scoreRates);

                    // 累加班级整体得分率
                    for (Map.Entry<String, Double> entry : scoreRates.entrySet()) {
                        classScoreRates.put(entry.getKey(), classScoreRates.getOrDefault(entry.getKey(), 0.0) + entry.getValue());
                    }
                }
            }
        } else {
            // 无考试数据时，初始化所有学生的默认分数为60
            if (groupList != null) {
                for (RMember member : groupList.members) {
                    String studentId = member.getId();
                    String studentName = studentIdToName.getOrDefault(studentId, "未知学生");
                    Map<String, Double> defaultScores = new HashMap<>();
                    for (String block : knowledgeBlocks) {
                        defaultScores.put(block, 60.0); // 初始化为60分
                    }
                    studentScoreRates.put(studentName, defaultScores);
                }
            }
        }

        // 2. 获取班级所有学生的主观分数（次数）
        List<Map<String, Object>> subjectiveScoresList = calculateScoresWithDetails(res, appraise);

        // 3. 将主观次数转换为0-100分数
        Map<String, Map<String, Double>> subjectiveScores = new HashMap<>();
        for (Map<String, Object> studentScore : subjectiveScoresList) {
            String studentId = (String) studentScore.get("studentId");
            @SuppressWarnings("unchecked")
            Map<String, Integer> scores = (Map<String, Integer>) studentScore.get("scores");
            Map<String, Double> convertedScores = new HashMap<>();

            for (Map.Entry<String, Integer> entry : scores.entrySet()) {
                String block = entry.getKey();
                int count = entry.getValue();
                // 确保 count 不小于0
                if (count < 0) {
                    count = 0;
                }
                double maxCount = 50.0; // 调整此参数控制分数增长速率

                // 使用反比例函数计算加分
                double additionalScore = 40.0 * ((double) count / (count + maxCount));
                double convertedScore = 60.0 + additionalScore;

                // 保留两位小数
                convertedScore = Double.parseDouble(String.format("%.2f", convertedScore));
                convertedScores.put(block, convertedScore);
            }

            subjectiveScores.put(studentId, convertedScores);
        }

        // 4. 融合主观和客观分数
        Map<String, Map<String, Double>> compositeStudentScores = new HashMap<>();
        for (String studentId : studentIdToName.keySet()) {
            String studentName = studentIdToName.get(studentId);
            // 获取客观分数（已确保60-100分）
            Map<String, Double> objective = studentScoreRates.getOrDefault(studentName, new HashMap<>());
            // 获取主观分数（确保60-100分）
            Map<String, Double> subjective = subjectiveScores.getOrDefault(studentId, new HashMap<>());

            Map<String, Double> compositeScores = new HashMap<>();
            for (String block : knowledgeBlocks) {
                // 客观分数兜底逻辑（确保60分）
                double objectiveScore = objective.getOrDefault(block, 60.0);
                // 主观分数兜底逻辑（确保60分）
                double subjectiveScore = subjective.getOrDefault(block, 60.0);

                // 计算综合得分（主观60% + 客观40%）
                double compositeScore = (subjectiveScore * 0.6) + (objectiveScore * 0.4);
                compositeScore = Math.min(100.0, compositeScore);  // 确保不超过100分
                compositeScore = Double.parseDouble(String.format("%.2f", compositeScore));

                compositeScores.put(block, compositeScore);
            }
            compositeStudentScores.put(studentName, compositeScores);
        }

        // 5. 计算班级整体得分率
        int studentCount = compositeStudentScores.size();
        Map<String, Double> compositeClassScoreRates = new HashMap<>();
        if (studentCount > 0) {
            for (Map<String, Double> studentScores : compositeStudentScores.values()) {
                for (String block : knowledgeBlocks) {
                    double score = studentScores.getOrDefault(block, 0.0);
                    compositeClassScoreRates.put(block, compositeClassScoreRates.getOrDefault(block, 0.0) + score);
                }
            }
            compositeClassScoreRates.replaceAll((k, v) -> Double.parseDouble(String.format("%.2f", v / studentCount)));
        } else {
            // 无学生时初始化空值
            for (String block : knowledgeBlocks) {
                compositeClassScoreRates.put(block, 0.0);
            }
        }

        // 6. 初始化所有同层的父节点
        for (AppraiseTreeNode node : appraise.getNodes()) {
            if (node.getName().equals("劳育")) {
                for (AppraiseTreeNode child : node.getChildren()) {
                    if (!compositeClassScoreRates.containsKey(child.getName())) {
                        compositeClassScoreRates.put(child.getName(), 0.0);
                    }
                }
            }
        }

        // 7. 转换 classScoreRates 和 studentScoreRates 为新的数据结构
        return getResult(className, compositeClassScoreRates, compositeStudentScores);
    }

    private static @NotNull Map<String, Object> getResult(String className, Map<String, Double> compositeClassScoreRates, Map<String, Map<String, Double>> compositeStudentScores) {
        List<Map<String, Object>> adjustedClassScoreRates = new ArrayList<>();
        Map<String, Object> classScore = new HashMap<>();
        classScore.put("className", className); // 设置班级名称

        List<Map<String, Object>> classBlocks = new ArrayList<>();
        for (Map.Entry<String, Double> entry : compositeClassScoreRates.entrySet()) {
            Map<String, Object> block = new HashMap<>();
            block.put("name", entry.getKey());
            block.put("score", entry.getValue());
            classBlocks.add(block);
        }
        classScore.put("blocks", classBlocks);
        adjustedClassScoreRates.add(classScore);

        List<Map<String, Object>> adjustedStudentScoreRates = new ArrayList<>();
        for (Map.Entry<String, Map<String, Double>> entry : compositeStudentScores.entrySet()) {
            String studentId = entry.getKey();
            Map<String, Double> scores = entry.getValue();

            Map<String, Object> studentScore = new HashMap<>();
            studentScore.put("studentName", studentId); // 设置学生名称

            List<Map<String, Object>> studentBlocks = new ArrayList<>();
            for (Map.Entry<String, Double> scoreEntry : scores.entrySet()) {
                Map<String, Object> block = new HashMap<>();
                block.put("name", scoreEntry.getKey());
                block.put("score", scoreEntry.getValue());
                studentBlocks.add(block);
            }
            studentScore.put("blocks", studentBlocks);
            adjustedStudentScoreRates.add(studentScore);
        }

        // 8. 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("classScoreRates", adjustedClassScoreRates); // 班级整体得分率
        result.put("studentScoreRates", adjustedStudentScoreRates); // 每个学生的综合得分率
        return result;
    }


    private static List<LaborEducationServiceImpl.KnowledgeScoreRate> calculateStudentScoreRates(
            String studentId,
            List<ExamClassResult> examResults,
            Map<String, List<List<String>>> knowledgeMap,
            Map<String, List<Double>> points,
            Map<String, Double> knowledgeTotalScore) {

        Map<String, Double> knowledgeTotalAverage = new HashMap<>();
        Map<String, Integer> knowledgeExamCount = new HashMap<>();

        // 如果没有考试数据，初始化所有知识点分数为60分
        if (examResults == null || examResults.isEmpty()) {
            for (String knowledge : knowledgeMap.values().stream()
                    .flatMap(List::stream)
                    .flatMap(List::stream)
                    .collect(Collectors.toSet())) {
                knowledgeTotalScore.put(knowledge, 60.0); // 默认60分
            }
            return knowledgeTotalScore.entrySet().stream()
                    .map(entry -> new LaborEducationServiceImpl.KnowledgeScoreRate(entry.getKey(), entry.getValue()))
                    .collect(Collectors.toList());
        }

        // 正常处理考试数据
        for (ExamClassResult examResult : examResults) {
            int studentIndex = examResult.getStudentIds().indexOf(studentId);
            if (studentIndex == -1) continue;
            int status = examResult.getStatus().get(studentIndex);
            if (status == 1) continue;

            List<List<String>> knowledgeList = knowledgeMap.get(examResult.getExamId());
            List<Double> pointList = points.get(examResult.getExamId());
            if (knowledgeList == null || pointList == null) continue;

            List<List<Double>> studentScores = examResult.getStudentScores();
            if (studentIndex >= studentScores.size() || studentScores.get(studentIndex) == null) continue;
            List<Double> studentScoreList = studentScores.get(studentIndex);

            if (knowledgeList.size() != studentScoreList.size()) continue;

            Map<String, Double> currentExamKnowledgeTotal = new HashMap<>();
            Map<String, Integer> currentExamKnowledgeCount = new HashMap<>();

            // 遍历每道题
            for (int i = 0; i < studentScoreList.size(); i++) {
                List<String> knowledgePoints = knowledgeList.get(i); // 题目对应的知识点列表
                double score = studentScoreList.get(i);

                // 如果题目没有知识点或得分为空，跳过
                if (knowledgePoints.isEmpty() || score < 0) continue;

                // 将题目得分分摊到每个知识点
                double scorePerKnowledge = score / knowledgePoints.size();

                // 遍历该题的所有知识点
                for (String knowledge : knowledgePoints) {
                    // 累加当前考试中该知识点的总得分和题目数量
                    currentExamKnowledgeTotal.merge(knowledge, scorePerKnowledge, Double::sum);
                    currentExamKnowledgeCount.merge(knowledge, 1, Integer::sum);
                }
            }

            // 计算当前考试中每个知识点的平均分（总得分/题目数量）
            for (String knowledge : currentExamKnowledgeTotal.keySet()) {
                double total = currentExamKnowledgeTotal.get(knowledge);
                int count = currentExamKnowledgeCount.get(knowledge);
                double average = total / count;

                // 累加跨考试的总平均分和考试次数
                knowledgeTotalAverage.merge(knowledge, average, Double::sum);
                knowledgeExamCount.merge(knowledge, 1, Integer::sum);
            }
        }

        // 计算最终平均分（所有考试平均分的平均），并限制在60-100分之间
        for (String knowledge : knowledgeTotalAverage.keySet()) {
            double totalAverage = knowledgeTotalAverage.get(knowledge);
            int examCount = knowledgeExamCount.get(knowledge);
            double finalAverage = totalAverage / examCount;

            // 在基础分60分上累加实际考试得分（最多加40分）
            double additionalScore = Math.min(finalAverage, 40.0); // 实际考试得分最多加40分
            finalAverage = 60.0 + additionalScore; // 基础分60 + 实际考试得分
            finalAverage = Math.min(finalAverage, 100.0); // 总分不超过100分

            finalAverage = Double.parseDouble(String.format("%.2f", finalAverage));
            knowledgeTotalScore.put(knowledge, finalAverage);
        }

        // 如果没有考试数据，确保所有知识点都有默认60分
        if (knowledgeTotalScore.isEmpty()) {
            for (String knowledge : knowledgeMap.values().stream()
                    .flatMap(List::stream)
                    .flatMap(List::stream)
                    .collect(Collectors.toSet())) {
                knowledgeTotalScore.put(knowledge, 60.0); // 默认60分
            }
        }

        return knowledgeTotalScore.entrySet().stream()
                .map(entry -> new LaborEducationServiceImpl.KnowledgeScoreRate(entry.getKey(), entry.getValue()))
                .collect(Collectors.toList());
    }


    public static double calculateNodeScoreRate(AppraiseTreeNode node, Map<String, Double> knowledgeTotalScore) {
        // 获取当前节点的得分
        double totalScore = knowledgeTotalScore.getOrDefault(node.getName(), 0.0);

        // 递归计算子节点的得分并累加
        for (AppraiseTreeNode child : node.getChildren()) {
            totalScore += calculateNodeScoreRate(child, knowledgeTotalScore);
        }

        return totalScore;
    }

    /**
     * 计算年级整体得分率，并返回其所属节点的父节点得分率
     *
     * @param gradeId         年级 ID
     * @param examResults     学生作答结果集
     * @param knowledgeMap    知识点映射（试卷 ID -> 知识点列表）
     * @param appraise        评价对象
     *  @param points         试卷配分
     * @return 年级所属节点的父节点得分率
     */
    public static Map<String, Object> calculateKnowledgeScoreForGrade(
            String gradeId,
            List<ExamClassResult> examResults,
            Map<String, List<List<String>>> knowledgeMap,
            Appraise appraise,
            Map<String, List<Double>> points,
            List<RecordVo> res,
            List<GradeAndClassVo> gradeAndClassVos) {

        // 1. 获取年级所有班级的客观分数
        Map<String, Map<String, Double>> classScores = new HashMap<>();
        Map<String, Double> gradeScores = new HashMap<>();

        // 根据 gradeId 匹配并取出 classId 集合
        List<GradeAndClassVo.CI> classInfos = gradeAndClassVos.stream()
                .filter(gradeAndClassVo -> String.valueOf(gradeAndClassVo.getGradeId()).equals(gradeId))
                .flatMap(gradeAndClassVo -> gradeAndClassVo.getClasses().stream())
                .collect(Collectors.toList());
        String gradeName = gradeAndClassVos.stream()
                .filter(gradeAndClassVo -> String.valueOf(gradeAndClassVo.getGradeId()).equals(gradeId))
                .findFirst()
                .map(GradeAndClassVo::getGradeName)
                .orElse("");

        // 过滤出该年级的考试结果
        List<ExamClassResult> gradeExamResults = examResults.stream()
                .filter(examResult -> examResult.getGradeId().equals(gradeId))
                .collect(Collectors.toList());

        // 2. 获取年级所有学生的主观分数（次数）
        List<Map<String, Object>> subjectiveScoresList = calculateScoresWithDetails(res, appraise);

        // 3. 将主观次数转换为0-100分数，并收集学生班级信息
        Map<String, Map<String, Double>> subjectiveScores = new HashMap<>();
        Map<String, String> studentClassMap = new HashMap<>(); // 学生ID到班级ID的映射
        for (RecordVo record : res) {
            // 假设RecordVo中有getStudentId()和getClassId()方法
            studentClassMap.put(record.getTargetId(), record.getClassId());
        }
        for (Map<String, Object> studentScore : subjectiveScoresList) {
            String studentId = (String) studentScore.get("studentId");
            @SuppressWarnings("unchecked")
            Map<String, Integer> scores = (Map<String, Integer>) studentScore.get("scores");
            Map<String, Double> convertedScores = new HashMap<>();

            for (Map.Entry<String, Integer> entry : scores.entrySet()) {
                String block = entry.getKey();
                int count = entry.getValue();
                // 确保 count 不小于0
                if (count < 0) {
                    count = 0;
                }
                double maxCount = 50.0; // 调整此参数控制分数增长速率

                // 使用反比例函数计算加分
                double additionalScore = 40.0 * ((double) count / (count + maxCount));
                double convertedScore = 60.0 + additionalScore;

                // 保留两位小数
                convertedScore = Double.parseDouble(String.format("%.2f", convertedScore));
                convertedScores.put(block, convertedScore);
            }
            subjectiveScores.put(studentId, convertedScores);
        }

        // 4. 遍历年级下的所有班级
        for (GradeAndClassVo.CI info : classInfos) {
            String classId = info.getClassId();
            String className = info.getClassName();
            Map<String, Double> classScoresInner = new HashMap<>();

            // 获取该班级的所有学生ID（来自res记录）
            List<String> studentIdsInClass = studentClassMap.entrySet().stream()
                    .filter(entry -> entry.getValue().equals(classId))
                    .map(Map.Entry::getKey)
                    .collect(Collectors.toList());

            Map<String, Double> classScoreSum = new HashMap<>();
            Map<String, Integer> classScoreCount = new HashMap<>();

            for (String studentId : studentIdsInClass) {
                // 获取学生状态（从考试结果中查找，默认为有效）
                int status = 0;
                for (ExamClassResult examResult : gradeExamResults) {
                    int index = examResult.getStudentIds().indexOf(studentId);
                    if (index != -1) {
                        status = examResult.getStatus().get(index);
                        break;
                    }
                }
                if (status == 1) continue;

                // 计算学生的知识点得分（客观分数）
                Map<String, Double> studentObjectiveScores = calculateKnowledgeScoreRateForStudent(
                        studentId, examResults, knowledgeMap, appraise, points
                );

                // 确保客观分数最低为60分
                for (Map.Entry<String, Double> entry : studentObjectiveScores.entrySet()) {
                    String block = entry.getKey();
                    double score = entry.getValue();
                    score = Math.max(60.0, Math.min(100.0, score)); // 确保60-100分
                    studentObjectiveScores.put(block, score);
                }

                // 获取学生的主观分数
                Map<String, Double> studentSubjectiveScores = subjectiveScores.getOrDefault(studentId, new HashMap<>());

                // 确定所有需要处理的评分块（合并主客观的块）
                Set<String> allBlocks = new HashSet<>();
                allBlocks.addAll(studentObjectiveScores.keySet());
                allBlocks.addAll(studentSubjectiveScores.keySet());

                // 计算综合得分（主观60%，客观40%）
                for (String block : allBlocks) {
                    double objectiveScore = studentObjectiveScores.getOrDefault(block, 60.0); // 默认60分
                    double subjectiveScore = studentSubjectiveScores.getOrDefault(block, 0.0);
                    double compositeScore = (subjectiveScore * 0.6) + (objectiveScore * 0.4);
                    compositeScore = Math.max(60.0, Math.min(100.0, compositeScore)); // 确保60-100分
                    compositeScore = Double.parseDouble(String.format("%.2f", compositeScore));

                    // 累加班级得分
                    classScoreSum.put(block, classScoreSum.getOrDefault(block, 0.0) + compositeScore);
                    classScoreCount.put(block, classScoreCount.getOrDefault(block, 0) + 1);
                }
            }

            // 计算班级平均得分
            for (String block : classScoreSum.keySet()) {
                double total = classScoreSum.get(block);
                int count = classScoreCount.getOrDefault(block, 1);
                classScoresInner.put(block, Double.parseDouble(String.format("%.2f", total / count)));
            }

            // 初始化所有同层的父节点（确保结构完整）
            for (AppraiseTreeNode node : appraise.getNodes()) {
                if ("劳育".equals(node.getName())) {
                    node.getChildren().forEach(child ->
                            classScoresInner.putIfAbsent(child.getName(), 60.0)); // 默认60分
                }
            }

            classScores.put(className, classScoresInner);
        }

        // 5. 计算年级的平均得分
        Map<String, Double> gradeScoreSum = new HashMap<>();
        Map<String, Integer> gradeScoreCount = new HashMap<>();
        classScores.values().forEach(classScore ->
                classScore.forEach((block, score) -> {
                    gradeScoreSum.put(block, gradeScoreSum.getOrDefault(block, 0.0) + score);
                    gradeScoreCount.put(block, gradeScoreCount.getOrDefault(block, 0) + 1);
                }));

        gradeScoreSum.forEach((block, total) -> {
            int count = gradeScoreCount.getOrDefault(block, 1);
            gradeScores.put(block, Double.parseDouble(String.format("%.2f", total / count)));
        });

        // 初始化年级层级的父节点
        for (AppraiseTreeNode node : appraise.getNodes()) {
            if ("劳育".equals(node.getName())) {
                node.getChildren().forEach(child ->
                        gradeScores.putIfAbsent(child.getName(), 60.0)); // 默认60分
            }
        }

        // 6. 转换数据结构并返回
        return getStringObjectMap(gradeName, gradeScores, classScores);
    }

    private static @NotNull Map<String, Object> getStringObjectMap(String gradeName, Map<String, Double> gradeScores, Map<String, Map<String, Double>> classScores) {
        List<Map<String, Object>> adjustedGradeScores = new ArrayList<>();
        Map<String, Object> gradeScore = new HashMap<>();
        gradeScore.put("gradeName", gradeName); // 设置年级名称

        List<Map<String, Object>> gradeBlocks = new ArrayList<>();
        for (Map.Entry<String, Double> entry : gradeScores.entrySet()) {
            Map<String, Object> block = new HashMap<>();
            block.put("name", entry.getKey());
            block.put("score", entry.getValue());
            gradeBlocks.add(block);
        }
        gradeScore.put("blocks", gradeBlocks);
        adjustedGradeScores.add(gradeScore);

        List<Map<String, Object>> adjustedClassScores = new ArrayList<>();
        for (Map.Entry<String, Map<String, Double>> entry : classScores.entrySet()) {
            String className = entry.getKey();
            Map<String, Double> scores = entry.getValue();

            Map<String, Object> classScore = new HashMap<>();
            classScore.put("className", className); // 设置班级名称

            List<Map<String, Object>> classBlocks = new ArrayList<>();
            for (Map.Entry<String, Double> scoreEntry : scores.entrySet()) {
                Map<String, Object> block = new HashMap<>();
                block.put("name", scoreEntry.getKey());
                block.put("score", scoreEntry.getValue());
                classBlocks.add(block);
            }
            classScore.put("blocks", classBlocks);
            adjustedClassScores.add(classScore);
        }

        // 7. 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("gradeScores", adjustedGradeScores); // 年级平均得分
        result.put("classScores", adjustedClassScores); // 每个班级的平均得分
        return result;
    }


    public static Map<String, Object> calculateKnowledgeScoreForSchool(
            List<ExamClassResult> examResults,
            Map<String, List<List<String>>> knowledgeMap,
            Appraise appraise,
            Map<String, List<Double>> points,
            List<RecordVo> res,
            List<School.Period> period,
            List<GradeAndClassVo> gradeAndClassVos) {

        // 1. 初始化数据结构
        Map<String, Map<String, Double>> gradeScores = new HashMap<>();
        Map<String, Double> schoolScores = new HashMap<>();
        List<String> gradeIds = period.stream()
                .flatMap(period1 -> period1.getGrades().stream())
                .collect(Collectors.toList());
        int totalClassCount = 0; // 新增：全校总班级数统计

        // 2. 遍历所有年级
        int index = 0;
        for (String gradeId : gradeIds) {
            Map<String, Object> gradeResult = calculateKnowledgeScoreForGrade(
                    String.valueOf(index), examResults, knowledgeMap, appraise, points, res, gradeAndClassVos
            );

            // 2.1 提取年级平均分
            List<Map<String, Object>> adjustedGradeScores = (List<Map<String, Object>>) gradeResult.get("gradeScores");
            Map<String, Double> gradeScoresInner = new HashMap<>();
            for (Map<String, Object> gradeScoreEntry : adjustedGradeScores) {
                List<Map<String, Object>> blocks = (List<Map<String, Object>>) gradeScoreEntry.get("blocks");
                for (Map<String, Object> block : blocks) {
                    String nodeName = (String) block.get("name");
                    Double score = (Double) block.get("score");
                    gradeScoresInner.put(nodeName, score);
                }
            }
            gradeScores.put(gradeId, gradeScoresInner);

            // 2.2 累加全校总分并统计班级数
            List<Map<String, Object>> adjustedClassScores = (List<Map<String, Object>>) gradeResult.get("classScores");
            totalClassCount += adjustedClassScores.size(); // 累加班级数
            for (Map<String, Object> classScoreEntry : adjustedClassScores) {
                List<Map<String, Object>> classBlocks = (List<Map<String, Object>>) classScoreEntry.get("blocks");
                for (Map<String, Object> block : classBlocks) {
                    String nodeName = (String) block.get("name");
                    Double score = (Double) block.get("score");
                    schoolScores.put(nodeName, schoolScores.getOrDefault(nodeName, 0.0) + score);
                }
            }
            index++;
        }

        // 3. 计算全校平均分
        if (gradeIds.size() == 1) {
            schoolScores.putAll(gradeScores.get(gradeIds.iterator().next()));
        } else {
            for (Map.Entry<String, Double> entry : schoolScores.entrySet()) {
                String nodeName = entry.getKey();
                double totalScore = entry.getValue();
                if (totalClassCount == 0) {
                    schoolScores.put(nodeName, 0.0);
                } else {
                    double average = totalScore / totalClassCount;
                    average = Double.parseDouble(String.format("%.2f", average));
                    schoolScores.put(nodeName, average);
                }
            }
        }

        // 4. 初始化未参与计算的父节点（保持默认0分）
        for (AppraiseTreeNode node : appraise.getNodes()) {
            if (node.getName().equals("劳育")) {
                for (AppraiseTreeNode child : node.getChildren()) {
                    if (!schoolScores.containsKey(child.getName())) {
                        schoolScores.put(child.getName(), 0.0);
                    }
                }
            }
        }

        // 5. 转换数据结构并返回
        return getResult(gradeScores, schoolScores);
    }

    private static @NotNull Map<String, Object> getResult(Map<String, Map<String, Double>> gradeScores, Map<String, Double> schoolScores) {
        List<Map<String, Object>> adjustedGradeScores = new ArrayList<>();
        for (Map.Entry<String, Map<String, Double>> entry : gradeScores.entrySet()) {
            String gradeName = entry.getKey();
            Map<String, Double> scores = entry.getValue();

            List<Map<String, Object>> blocks = new ArrayList<>();
            for (Map.Entry<String, Double> scoreEntry : scores.entrySet()) {
                Map<String, Object> block = new HashMap<>();
                block.put("name", scoreEntry.getKey());
                block.put("score", scoreEntry.getValue());
                blocks.add(block);
            }

            Map<String, Object> gradeScore = new HashMap<>();
            gradeScore.put("gradeName", gradeName);
            gradeScore.put("blocks", blocks);
            adjustedGradeScores.add(gradeScore);
        }

        // 8. 转换 schoolScores 为新的数据结构
        List<Map<String, Object>> adjustedSchoolScores = new ArrayList<>();
        Map<String, Object> schoolScore = new HashMap<>();
        schoolScore.put("gradeName", "全校"); // 全校的 gradeName 设置为 "全校"

        List<Map<String, Object>> schoolBlocks = new ArrayList<>();
        for (Map.Entry<String, Double> entry : schoolScores.entrySet()) {
            Map<String, Object> block = new HashMap<>();
            block.put("name", entry.getKey());
            block.put("score", entry.getValue());
            schoolBlocks.add(block);
        }
        schoolScore.put("blocks", schoolBlocks);
        adjustedSchoolScores.add(schoolScore);

        // 9. 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("schoolScores", adjustedSchoolScores); // 全校平均得分
        result.put("gradeScores", adjustedGradeScores); // 每个年级的平均得分
        return result;
    }


    /**
     * 查找知识点对应的节点
     *
     * @param nodes             节点列表
     * @param knowledge 学生知识点得分率列表
     * @return 知识点对应的节点
     */
    private static AppraiseTreeNode findKnowledgeNode(List<AppraiseTreeNode> nodes, String knowledge) {
        for (AppraiseTreeNode node : nodes) {
            if (node.getName().equals("劳育")) {
                for (AppraiseTreeNode child : node.getChildren()) {
                    for (AppraiseTreeNode grandchild : child.getChildren()) {
                        if (grandchild.getName().equals(knowledge)) {
                            return grandchild; // 返回知识点对应的节点
                        }
                    }
                }
            }
        }
        return null; // 未找到匹配的节点
    }

    private static AppraiseTreeNode findParentNode(List<AppraiseTreeNode> nodes, String nodeId) {
        for (AppraiseTreeNode node : nodes) {
            for (AppraiseTreeNode child : node.getChildren()) {
                for (AppraiseTreeNode grandchild : child.getChildren()) {
                    if (grandchild.getId().equals(nodeId)) {
                        return child; // 返回父节点
                    }
                }
            }
        }
        return null; // 未找到父节点
    }

}
